@@ -279,9 +279,8 @@ internet_address_to_string (InternetAddress *ia, GMimeFormatOptions *options, gb
 	
 	string = g_string_new ("");
 	INTERNET_ADDRESS_GET_CLASS (ia)->to_string (ia, options, flags, &linelen, string);
-	str = string->str;
 	
-	g_string_free (string, FALSE);
+	str = g_string_free (string, FALSE);
 	
 	return str;
 }
@@ -1893,8 +1892,10 @@ address_parse (GMimeParserOptions *options, AddressParserFlags flags, const char
 		if (*inptr == '(') {
 			const char *comment = inptr;
 			
-			if (!skip_comment (&inptr))
+			if (!skip_comment (&inptr)) {
+				g_free (addrspec);
 				goto error;
+			}
 			
 			comment++;
 			
@@ -1904,8 +1905,10 @@ address_parse (GMimeParserOptions *options, AddressParserFlags flags, const char
 		}
 		
 		if (*inptr == '>') {
-			if (mode != GMIME_RFC_COMPLIANCE_LOOSE)
+			if (mode != GMIME_RFC_COMPLIANCE_LOOSE) {
+				g_free (addrspec);
 				goto error;
+			}
 			
 			inptr++;
 		}
@@ -1969,8 +1972,10 @@ address_parse (GMimeParserOptions *options, AddressParserFlags flags, const char
 		if (*inptr == '(') {
 			const char *comment = inptr;
 			
-			if (!skip_comment (&inptr))
+			if (!skip_comment (&inptr)) {
+				g_free (addrspec);
 				goto error;
+			}
 			
 			comment++;
 			
@@ -1999,8 +2004,11 @@ address_parse (GMimeParserOptions *options, AddressParserFlags flags, const char
 			 * is an unquoted string with an '@'. */
 			const char *end;
 			
-			if (mode != GMIME_RFC_COMPLIANCE_LOOSE)
+			if (mode != GMIME_RFC_COMPLIANCE_LOOSE) {
+				g_free (addrspec);
+				g_free (name);
 				goto error;
+			}
 			
 			end = inptr;
 			while (end > start && is_lwsp (*(end - 1)))
@@ -2016,8 +2024,11 @@ address_parse (GMimeParserOptions *options, AddressParserFlags flags, const char
 			 * anyway in order to deal with the second Unbalanced Angle Brackets example in
 			 * section 7.1.3: second@example.org> */
 			if (*inptr == '>') {
-				if (mode != GMIME_RFC_COMPLIANCE_LOOSE)
+				if (mode != GMIME_RFC_COMPLIANCE_LOOSE) {
+					g_free (addrspec);
+					g_free (name);
 					goto error;
+				}
 				
 				inptr++;
 			}
@@ -2178,6 +2189,7 @@ _internet_address_list_append_parse (InternetAddressList *list, GMimeParserOptio
 /**
  * internet_address_list_append_parse:
  * @list: a #InternetAddressList
+ * @options: (nullable): a #GMimeParserOptions or %NULL
  * @str: a string containing internet addresses
  *
  * Adds all of the addresses in @str to @list.
